@@ -2,7 +2,7 @@ import ismrmrd
 import numpy as np
 import torch 
 from torch import fft as FFT
-from mrftools import SequenceParameters, DictionaryParameters, Simulation, SequenceType
+from mrftools import SequenceParameters, DictionaryParameters, Simulation
 import time
 import torchkbnufft as tkbn
 from tqdm import tqdm
