@@ -110,3 +110,26 @@ class BitReverse:
     def Generate(numValues=960, max=48, firstValue=0):
         br = BitReverse()
         return br.__generate(numValues, max, firstValue)
+
+class ScaledRectifiedSinusoid:
+    def __generate(self, numValues, lobeAmplitudeList, minimum, patternInitialPhase):
+        numLobes = len(lobeAmplitudeList)
+        lobeLength = numValues / numLobes
+        degreesPerTimepoint = np.pi/lobeLength
+        timepointPatternAngles = np.arange(0,np.pi*numLobes,degreesPerTimepoint)
+        timepointPatternScalars = np.abs(np.sin(timepointPatternAngles))
+        timepointAmplitudeScalars = np.array([])
+        for lobeAmplitude in lobeAmplitudeList:
+            for lobeTimepoint in np.arange(0,lobeLength):
+                timepointAmplitudeScalars = np.append(timepointAmplitudeScalars,lobeAmplitude)
+        timepointPatternScalars = np.roll(timepointPatternScalars,-1*int(patternInitialPhase/degreesPerTimepoint))
+        timepointAmplitudeScalars = np.roll(timepointAmplitudeScalars,-1*int(patternInitialPhase/degreesPerTimepoint))
+        values = timepointPatternScalars * timepointAmplitudeScalars
+        values = values[0:numValues] 
+        values[values<minimum] = minimum
+        return values
+
+    @staticmethod
+    def Generate(numValues, lobeAmplitudeList, minimum=0, patternInitialPhase=0):
+        s = ScaledRectifiedSinusoid()
+        return s.__generate(numValues, lobeAmplitudeList, minimum, patternInitialPhase)
\ No newline at end of file
