@@ -7,110 +7,34 @@ from tqdm import tqdm
 from mrftools import DictionaryParameters, SequenceParameters
 
 class Simulation: 
-    def __init__(self,sequenceParameters, dictionaryParameters, name="", numTimepoints = -1, phaseRange=(-2*np.pi, 2*np.pi), numSpins=1, results=[], truncationMatrix=[], truncatedResults=[], singularValues=[], initialMz=-0.95):
+    def __init__(self,sequenceParameters, dictionaryParameters, name="", numSpins=1, times=[], timeDomainResults=[], results=[], truncationMatrix=[], truncatedResults=[], singularValues=[]):
         self.sequenceParameters = sequenceParameters
         self.dictionaryParameters = dictionaryParameters
-        if numTimepoints == -1:
-            numTimepoints = len(sequenceParameters.timepoints)
-        self.numTimepoints = numTimepoints
-        self.phaseRange = phaseRange
         self.numSpins = numSpins
+        self.times = times
+        self.timeDomainResults = timeDomainResults
         self.results = results
         self.truncationMatrix = truncationMatrix
         self.truncatedResults = truncatedResults
         self.singularValues = singularValues
-        self.initialMz = initialMz
         if not name:
-            self.name = sequenceParameters.name + "_" + dictionaryParameters.name + "_" + str(numSpins) # Doesn't account for phase range
+            self.name = sequenceParameters.name + "_" + dictionaryParameters.name + "_" + str(numSpins)
         else:
             self.name = name
 
-    @staticmethod
-    def Simulate(numTimepoints,T1s,T2s,B1s,TRs,TEs,FAs,spinOffresonances,device=None, initialMz=-0.95):     
-        if(device==None):
-            if torch.cuda.is_available():
-                device = torch.device("cuda")
-            else:
-                device = torch.device("cpu")    
-
-        T1s = torch.tensor(T1s).to(device)
-        T2s = torch.tensor(T2s).to(device)
-        B1s = torch.tensor(B1s).to(device)
-        FAs = torch.tensor(FAs).to(device)
-        spinOffresonances = torch.tensor(spinOffresonances).to(device)
-        Mx0 = torch.zeros((numTimepoints,len(spinOffresonances),len(T1s)))
-        My0 = torch.zeros((numTimepoints,len(spinOffresonances),len(T1s)))
-        Mz0 = torch.zeros((numTimepoints,len(spinOffresonances),len(T1s)))
-        Mx = torch.zeros((len(spinOffresonances),len(T1s))).to(device)
-        My = torch.zeros((len(spinOffresonances),len(T1s))).to(device)
-        Mz = torch.zeros((len(spinOffresonances),len(T1s))).to(device)
-
-        FAs = torch.deg2rad(FAs)
-        phaseValueCosines = torch.cos(spinOffresonances)
-        phaseValueSines = torch.sin(spinOffresonances)
-        
-        Mz[:] = initialMz
-        with torch.no_grad():
-            for iTimepoint in range(numTimepoints):
-                fa = FAs[iTimepoint] * B1s
-                tr = TRs[iTimepoint]
-                te = TEs[iTimepoint]
-                tre = tr-te
-                
-                At2te = torch.exp(-1*te/T2s)
-                At1te = torch.exp(-1*te/T1s)
-                Bt1te = 1-At1te
-                
-                At2tr = torch.exp(-1*tre/T2s)
-                At1tr = torch.exp(-1*tre/T1s)
-                Bt1tr = 1-At1tr
-            
-                crf = torch.cos(fa)
-                srf = torch.sin(fa)
-
-                # Applying flip angle 
-                Myi = My
-                Mzi = Mz
-                My = torch.multiply(crf,Myi)-torch.multiply(srf,Mzi)
-                Mz = torch.multiply(srf,Myi)+torch.multiply(crf,Mzi)
-
-                # Relaxation over TE
-                Mx = torch.multiply(Mx, At2te)
-                My = torch.multiply(My, At2te)
-                Mz = torch.multiply(Mz, At1te)+Bt1te
-
-                # Reading value after TE and before TRE 
-                Mx0[iTimepoint,:,:]=Mx.cpu()
-                My0[iTimepoint,:,:]=My.cpu()
-                Mz0[iTimepoint,:,:]=Mz.cpu()
-
-                # Relaxation over TRE (TR-TE) 
-                Mx = Mx*At2tr
-                My = My*At2tr
-                Mz = Mz*At1tr+Bt1tr
-
-                # Applying off-resonance to spins
-                Mxi = Mx.t()
-                Myi = My.t()
-                Mx = (torch.multiply(phaseValueCosines,Mxi) - torch.multiply(phaseValueSines,Myi)).t()
-                My = (torch.multiply(phaseValueSines,Mxi) + torch.multiply(phaseValueCosines,Myi)).t()
-                del fa, tr, te, tre, At2te, At1te, Bt1te, At2tr, At1tr, Bt1tr, crf, srf, Mxi, Myi, Mzi
-        del T1s, T2s, B1s, FAs, spinOffresonances, Mx, My, Mz
-        return Mx0,My0,Mz0
-
     def Execute(self, numBatches=1, device=None):
         if(device==None):
             if torch.cuda.is_available():
                 device = torch.device("cuda")
             else:
                 device = torch.device("cpu")
-        TRs = np.copy(self.sequenceParameters.timepoints['TR'])
-        TEs = np.copy(self.sequenceParameters.timepoints['TE'])
-        FAs = np.copy(self.sequenceParameters.timepoints['FA'])
-        phaseValues = np.linspace(self.phaseRange[0], self.phaseRange[1], self.numSpins)
         dictEntriesPerBatch = int(len(self.dictionaryParameters.entries)/numBatches)
         print("Simulating " + str(numBatches) + " batch(s) of ~" + str(dictEntriesPerBatch) + " dictionary entries")
+        singleResult = self.sequenceParameters.Simulate(self.dictionaryParameters.entries[0], 1)
+        self.numTimepoints = np.shape(singleResult[1][0])[0]
+        self.numReadoutPoints = np.shape(singleResult[2][0])[0]
         Mxy = np.zeros((self.numTimepoints, len(self.dictionaryParameters.entries)), np.complex128)
+        ReadoutMxy = np.zeros((self.numReadoutPoints, len(self.dictionaryParameters.entries)), np.complex128)
         with tqdm(total=numBatches) as pbar:
             for i in range(numBatches):
                 firstDictEntry = i*dictEntriesPerBatch
@@ -118,16 +42,28 @@ class Simulation:
                     lastDictEntry = len(self.dictionaryParameters.entries)
                 else:
                     lastDictEntry = firstDictEntry+dictEntriesPerBatch
-                T1s = np.copy(self.dictionaryParameters.entries[firstDictEntry:lastDictEntry]['T1'])
-                T2s = np.copy(self.dictionaryParameters.entries[firstDictEntry:lastDictEntry]['T2'])
-                B1s = np.copy(self.dictionaryParameters.entries[firstDictEntry:lastDictEntry]['B1'])
-                Mx0,My0,Mz0 = self.Simulate(self.numTimepoints, T1s, T2s, B1s, TRs, TEs, FAs, phaseValues, device=device, initialMz=self.initialMz)
-                Mx = torch.mean(Mx0, axis=1)    
-                My = torch.mean(My0, axis=1)    
-                Mxy[:,firstDictEntry:lastDictEntry] = Mx+(My*1j)
+                batchDictionaryEntries = self.dictionaryParameters.entries[firstDictEntry:lastDictEntry]
+                allResults = self.sequenceParameters.Simulate(batchDictionaryEntries, self.numSpins, device=device)
+                Mx = torch.mean(allResults[1][0], axis=1)
+                My = torch.mean(allResults[1][1], axis=1)
+                Mxy[:,firstDictEntry:lastDictEntry] = Mx+(My*1j) 
+                ReadoutMx = torch.mean(allResults[2][0], axis=1)
+                ReadoutMy = torch.mean(allResults[2][1], axis=1)
+                ReadoutMxy[:,firstDictEntry:lastDictEntry] = ReadoutMx+(ReadoutMy*1j)
                 pbar.update(1)
-            self.results = Mxy
+        self.times = allResults[0]
+        self.timeDomainResults = Mxy
+        self.results = ReadoutMxy
         return self.results
+    
+    @staticmethod
+    def GetInnerProducts(querySignals, dictionarySignals):  
+        querySignalsTransposed = querySignals.transpose()
+        normalizedQuerySignals = querySignalsTransposed / np.linalg.norm(querySignalsTransposed, axis=1)[:,None]
+        simulationResultsTransposed = dictionarySignals.transpose()
+        normalizedSimulationResultsTransposed = simulationResultsTransposed / np.linalg.norm(simulationResultsTransposed, axis=1)[:,None]
+        innerProducts = np.inner(normalizedQuerySignals, normalizedSimulationResultsTransposed)
+        return innerProducts
 
     def CalculateSVD(self, desiredSVDPower=0.99, truncationNumberOverride=None):
         dictionary = self.results.transpose()
@@ -201,18 +137,20 @@ class Simulation:
         else:
             print("Input is not a .mrf file")
 
-    def Plot(self, numTimepoints=-1, dictionaryEntryNumbers=[], plotTruncated=False):
-        if numTimepoints == -1:
-            numTimepoints = len(self.sequenceParameters.timepoints)
+    def Plot(self, dictionaryEntryNumbers=[], plotTruncated=False, plotTimeDomain=False):
         if dictionaryEntryNumbers == []:
             dictionaryEntryNumbers = [int(len(self.dictionaryParameters.entries)/2)]
         ax = plt.subplot(1,1,1)
-        if not plotTruncated:
-            for entry in dictionaryEntryNumbers:
-                plt.plot(abs(self.results[:,entry]), label=str(self.dictionaryParameters.entries[entry]))
+        if not plotTimeDomain:
+            if not plotTruncated:
+                for entry in dictionaryEntryNumbers:
+                    plt.plot(abs(self.results[:,entry]), label=str(self.dictionaryParameters.entries[entry]))
+            else:
+                for entry in dictionaryEntryNumbers:
+                    plt.plot(abs(self.truncatedResults[:,entry]), label=str(self.dictionaryParameters.entries[entry]))
         else:
             for entry in dictionaryEntryNumbers:
-                plt.plot(abs(self.truncatedResults[:,entry]), label=str(self.dictionaryParameters.entries[entry]))
+                plt.plot(self.times, abs(self.timeDomainResults[:,entry]), label=str(self.dictionaryParameters.entries[entry]))
         ax.legend()
 
     def GetAverageResult(self, indices):
